// ***************************************
// Plugin I/O pin Class for SourceBoost IDE
// 
// Author(s): David Hobday
//
// Copyright (C) 2003-2004 Pavel Baranov
// Copyright (C) 2003-2004 David Hobday
// 
// This code may be freely used to develop
// Plugins for the SourceBoost IDE
// ***************************************

////////////////////////////////////////////
// Some very useful classes that encapsulate
// analog and digital IO pin functionality
////////////////////////////////////////////

#ifndef _IOPIN_H_
#define _IOPIN_H_

#include "PluginApi.h"
#include <iostream.h>
///
/// A class that represents a digital I/O pin
///
class CDigIoPin
{
public:
	CDigIoPin()
	{
		m_hConnection = NULL;

		m_outputLoR = 0;
		m_outputLoV = 0;
		m_outputHiR = 0;
		m_outputHiV = 0;
		m_inputR = 0;
		m_inputV = 0;
		m_inputLoV = 0;
		m_inputHiV = 0;

		m_pinV = 0;
		m_inState = 0;
		m_opState = 0;
		m_mode = INPUT;
	}

	///
	/// Configure the electrical characteristics of the Digital I/O Pin.
	///
	/// inputLoV - When in input mode, this is the logic 0 voltage level
	/// inputHiV - When in input mode, this is the logic 1 voltage level
	/// inputR - When in input mode, this is the load impendance applied to the MPU pin
	/// inputV - When in input mode, this the voltage applied to the other end of the load impendance
	/// outputLoV - When in output mode, this is the output low voltage for logic state 0
	/// outputLoR - When in output mode, this is the impendance of the output for logic state 0
	/// outputHiV - When in output mode, this is the output low voltage for logic state 1
	/// outputHiR - When in output mode, this is the impendance of the output for logic state 1
	///
	void Configure( int inputLoV, int inputHiV, int inputR, int inputV, int outputLoV, int outputLoR, int outputHiV, int outputHiR )
	{
		m_outputLoR = outputLoR;
		m_outputLoV = outputLoV;
		m_outputHiR = outputHiR;
		m_outputHiV = outputHiV;
		m_inputR = inputR;
		m_inputV = inputV;
		m_inputLoV = inputLoV;
		m_inputHiV = inputHiV;

		m_pinV = 0;
		m_inState = 0;
		m_opState = 0;
		m_mode = INPUT;
	}

	///
	/// Retrieves the underlying connection handle
	///
	HANDLE GetConnectionHandle() { return m_hConnection; }
	
	///
	/// Establishes a connection to a named MPU pin
	/// Previous connections should be disconnected before calling this function
	///
	bool Connect( const char* pinName )
	{
		MakeConnectionToPin( pinName, &m_hConnection );
		
		if ( m_hConnection == NULL )
			return false;

		if ( m_mode == INPUT )
			UpdateInput();
		else
			UpdateOutput();

		return true;
	}

	///
	/// Removes a previously established connection
	///
	void Disconnect()
	{
		if ( m_hConnection )
		{
			ReleaseConnection( m_hConnection );
			m_hConnection = NULL;
		}
	}

	///
	/// Makes the connection to a MPU pin use the output characteristics specified with configure
	///
	void MakeOutput()
	{
		if ( m_mode != OUTPUT )
		{
			m_mode = OUTPUT;				
			if ( m_hConnection == NULL )
				return;

			UpdateOutput();
		}
	}

	///
	/// Makes the connection to a MPU pin use the output characteristics specified with the Configure function
	///
	void MakeInput()
	{
		if ( m_mode != INPUT )
		{
			m_mode = INPUT;
			if ( m_hConnection == NULL )
				return;

			UpdateInput();
		}
	}

	///
	/// Sets the output voltage base on the specified digital I/O state,
	/// and the electrical characteristics specified with Configure function
	///
	void SetOutputState( int state )
	{
		if ( m_hConnection == NULL )
			return;

		// state has changed
		if ( m_opState != state )
		{
			m_opState = state;
			if ( m_mode == OUTPUT )				
				UpdateOutput();
		}
	}
	
	///
	/// Reads the pin voltage and returns the current logic state of the MPU Pin
	/// The logics state threshold voltages based on the electrical characteristics specified with Configure function
	///
	int ReadInputState()
	{
		if ( m_hConnection == NULL )
			return 0;

		GetPinV( m_hConnection, &m_pinV );
		if ( m_pinV > m_inputHiV )
			m_inState = 1;
		
		if ( m_pinV <= m_inputLoV )
			m_inState = 0;

		return m_inState;
	}

	///
	/// Saves the internal state to an ostream. Useful for implement SaveState functionality in a data class.
	///
	ostream& Save( ostream& os )
	{
		os.write( (const char*)&m_outputLoR, sizeof( m_outputLoR ) );
		os.write( (const char*)&m_outputLoV, sizeof( m_outputLoV ) );
		os.write( (const char*)&m_outputHiR, sizeof( m_outputHiR ) );
		os.write( (const char*)&m_outputHiV, sizeof( m_outputHiV ) );
		os.write( (const char*)&m_inputR, sizeof( m_inputR ) );
		os.write( (const char*)&m_inputV, sizeof( m_inputV ) );
		os.write( (const char*)&m_inputLoV, sizeof( m_inputLoV ) );
		os.write( (const char*)&m_inputHiV, sizeof( m_inputHiV ) );
		os.write( (const char*)&m_pinV, sizeof( m_pinV ) );
		os.write( (const char*)&m_inState, sizeof( m_inState ) );
		os.write( (const char*)&m_opState, sizeof( m_opState ) );
		os.write( (const char*)&m_mode, sizeof( m_mode ) );
		return os;
	}

	///
	/// Restores the internal state from an istream. Useful for implement LoadState functionality in a data class.
	///
	istream& Load( istream& is )
	{
		is.read( (char*)&m_outputLoR, sizeof( m_outputLoR ) );
		is.read( (char*)&m_outputLoV, sizeof( m_outputLoV ) );
		is.read( (char*)&m_outputHiR, sizeof( m_outputHiR ) );
		is.read( (char*)&m_outputHiV, sizeof( m_outputHiV ) );
		is.read( (char*)&m_inputR, sizeof( m_inputR ) );
		is.read( (char*)&m_inputV, sizeof( m_inputV ) );
		is.read( (char*)&m_inputLoV, sizeof( m_inputLoV ) );
		is.read( (char*)&m_inputHiV, sizeof( m_inputHiV ) );
		is.read( (char*)&m_pinV, sizeof( m_pinV ) );
		is.read( (char*)&m_inState, sizeof( m_inState ) );
		is.read( (char*)&m_opState, sizeof( m_opState ) );
		is.read( (char*)&m_mode, sizeof( m_mode ) );
		return is;
	}

private:
	void UpdateInput()
	{
			SetConnectionV( m_hConnection, m_inputV );
			SetConnectionR( m_hConnection, m_inputR );
	}
	
	void UpdateOutput()
	{
		if ( m_opState )
		{
			SetConnectionV( m_hConnection, m_outputHiV );
			SetConnectionR( m_hConnection, m_outputHiR );
		}
		else
		{
			SetConnectionV( m_hConnection, m_outputLoV );
			SetConnectionR( m_hConnection, m_outputLoR );
		}
	}

	enum { INPUT, OUTPUT };

	HANDLE m_hConnection;
	int m_outputLoR;
	int m_outputLoV;
	int m_outputHiR;
	int m_outputHiV;
	int m_inputR;
	int m_inputV;
	int m_inputLoV;
	int m_inputHiV;
	int m_pinV;
	int m_inState;
	int m_opState;
	int m_mode;
};



///
/// A class that represents an analog I/O pin
///
class CAnalogIoPin
{
public:
	CAnalogIoPin()
	{
		m_hConnection = NULL;

		m_inputV = 0;
		m_inputR = 0;
		m_outputR = 0;
		m_outputV = 0;

		m_pinV = 0;

		m_mode = INPUT;
	}


	///
	/// Configure the electrical characteristics of the Digital I/O Pin.
	///
	/// inputR - When in input mode, this is the load impendance applied to the MPU pin
	/// inputV - When in input mode, this is the logic 1 voltage level
	/// outputR - When in output mode, this is the impendance of the output
	///
	void Configure( int inputR, int inputV, int outputR )
	{
		m_inputV = inputV;
		m_inputR = inputR;
		m_outputR = outputR;
		
		m_pinV = 0;
		m_mode = INPUT;
	}

	///
	/// Retrieves the underlying connection handle
	///
	HANDLE GetConnectionHandle() { return m_hConnection; }

	///
	/// Establishes a connection to a named MPU pin
	/// Previous connections should be disconnected before calling this function
	///
	bool Connect( const char* pinName )
	{
		MakeConnectionToPin( pinName, &m_hConnection );
		
		if ( m_hConnection == NULL )
			return false;

		if ( m_mode == INPUT )
			UpdateInput();
		else
			UpdateOutput();

		return true;
	}

	///
	/// Removes a previously established connection
	///
	void Disconnect()
	{
		if ( m_hConnection )
		{
			ReleaseConnection( m_hConnection );
			m_hConnection = NULL;
		}
	}

	///
	/// Makes the connection to a MPU pin use the output characteristics specified with configure
	///
	void MakeOutput()
	{
		if ( m_mode != OUTPUT )
		{
			m_mode = OUTPUT;				
			if ( m_hConnection == NULL )
				return;

			UpdateOutput();
		}
	}

	///
	/// Makes the connection to a MPU pin use the output characteristics specified with the Configure function
	///
	void MakeInput()
	{
		if ( m_mode != INPUT )
		{
			m_mode = INPUT;
			if ( m_hConnection == NULL )
				return;

			UpdateInput();
		}
	}

	
	///
	/// Sets the output voltage.
	/// The electrical characteristics used are this specified with Configure function.
	///
	void SetOutputVoltage( int Vmv )
	{
		if ( m_hConnection == NULL )
			return;

		// state has changed
		if ( m_outputV != Vmv )
		{
			m_outputV = Vmv;
			if ( m_mode == OUTPUT )				
				UpdateOutput();
		}
	}
	
	///
	/// Reads the voltage on MPU pin to which a connection has been made
	///
	int ReadVoltage()
	{
		if ( m_hConnection == NULL )
			return 0;

		GetPinV( m_hConnection, &m_pinV );

		return m_pinV;
	}

	///
	/// Saves the internal state to an ostream. Useful for implement SaveState functionality in a data class.
	///
	ostream& Save( ostream& os )
	{
		os.write( (const char*)&m_inputV, sizeof( m_inputV ) );
		os.write( (const char*)&m_inputR, sizeof( m_inputR ) );
		os.write( (const char*)&m_outputR, sizeof( m_outputR ) );
		os.write( (const char*)&m_outputV, sizeof( m_outputV ) );
		os.write( (const char*)&m_pinV, sizeof( m_pinV ) );
		os.write( (const char*)&m_mode, sizeof( m_mode ) );
		return os;
	}

	///
	/// Restores the internal state from an istream. Useful for implement LoadState functionality in a data class.
	///
	istream& Load( istream& is )
	{
		is.read( (char*)&m_inputV, sizeof( m_inputV ) );
		is.read( (char*)&m_inputR, sizeof( m_inputR ) );
		is.read( (char*)&m_outputR, sizeof( m_outputR ) );
		is.read( (char*)&m_outputV, sizeof( m_outputV ) );
		is.read( (char*)&m_pinV, sizeof( m_pinV ) );
		is.read( (char*)&m_mode, sizeof( m_mode ) );
		return is;
	}

private:
	void UpdateInput()
	{
			SetConnectionV( m_hConnection, m_inputV );
			SetConnectionR( m_hConnection, m_inputR );
	}
	
	void UpdateOutput()
	{
		SetConnectionV( m_hConnection, m_outputV );
		SetConnectionR( m_hConnection, m_outputR );
	}

	enum { INPUT, OUTPUT };

	HANDLE m_hConnection;
	int m_inputV;
	int m_inputR;
	int m_outputR;
	int m_outputV;

	int m_pinV;

	int m_mode;
};


#endif _IOPIN_H_